@@ -27,6 +27,7 @@
 
 #include <algorithm>         // for reverse, sort
 #include <cstddef>           // for size_t
+#include <cstring>           // for strlen
 #include <initializer_list>  // for initializer_list
 #include <iterator>          // for distance
 #include <numeric>           // for accumulate
@@ -39,6 +40,7 @@
 
 #include "constants.hpp"    // for UNDEFINED
 #include "debug.hpp"        // for LIBSEMIGROUPS_ASSERT
+#include "int-range.hpp"    // for IntegralRange
 #include "order.hpp"        // for shortlex_compare
 #include "suffix-tree.hpp"  // for SuffixTree
 #include "uf.hpp"           // for Duf
@@ -56,7 +58,7 @@ namespace libsemigroups {
   //! the rules of the presentation. The Presentation class template also
   //! provide some checks that the rules really defines a presentation, (i.e.
   //! it's consistent with its alphabet), and some related functionality is
-  //! available in the namespace libsemigroups::presentation.
+  //! available in the namespace `libsemigroups::presentation`.
   //!
   //! \tparam W the type of the underlying words.
   template <typename W>
@@ -459,7 +461,7 @@ namespace libsemigroups {
       p.add_rule_and_check(lhop.begin(), lhop.end(), rhop.begin(), rhop.end());
     }
 
-    //! Add a rule to the presentation by char const pointer.
+    //! Add a rule to the presentation by `char const*`.
     //!
     //! Adds the rule with left hand side `lhop` and right hand side `rhop`
     //! to the rules.
@@ -483,7 +485,7 @@ namespace libsemigroups {
       add_rule(p, std::string(lhop), std::string(rhop));
     }
 
-    //! Add a rule to the presentation by char const pointer.
+    //! Add a rule to the presentation by `char const*`.
     //!
     //! Adds the rule with left hand side `lhop` and right hand side `rhop`
     //! to the rules.
@@ -504,7 +506,7 @@ namespace libsemigroups {
       add_rule_and_check(p, std::string(lhop), std::string(rhop));
     }
 
-    //! Add a rule to the presentation by initializer_list.
+    //! Add a rule to the presentation by `initializer_list`.
     //!
     //! Adds the rule with left hand side `lhop` and right hand side `rhop`
     //! to the rules.
@@ -529,7 +531,7 @@ namespace libsemigroups {
       p.add_rule(lhop.begin(), lhop.end(), rhop.begin(), rhop.end());
     }
 
-    //! Add a rule to the presentation by initializer_list.
+    //! Add a rule to the presentation by `initializer_list`.
     //!
     //! Adds the rule with left hand side `lhop` and right hand side `rhop`
     //! to the rules.
@@ -590,12 +592,29 @@ namespace libsemigroups {
     void add_identity_rules(Presentation<W>&                      p,
                             typename Presentation<W>::letter_type e);
 
+    //! Add rules for a zero element.
+    //!
+    //! Adds rules of the form \f$az = za = z\f$ for every letter \f$a\f$ in
+    //! the alphabet of \p p, and where \f$z\f$ is the second parameter.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //! \param p the presentation to add rules to
+    //! \param z the zero element
+    //!
+    //! \returns (None)
+    //!
+    //! \throws LibsemigroupsException if \p z is not a letter in
+    //! `p.alphabet()`.
+    template <typename W>
+    void add_zero_rules(Presentation<W>&                      p,
+                        typename Presentation<W>::letter_type z);
+
     //! Add rules for inverses.
     //!
     //! The letter in \c a with index \c i in \p vals is the inverse of the
-    //! letter in alphabet() with index \c i. The rules added are \f$a_ib_i =
-    //! e\f$ where the alphabet is \f$\{a_1, \ldots, a_n\}\f$; the parameter \p
-    //! vals is \f$\{b_1, \ldots, b_n\}\f$; and \f$e\f$ is the 3rd parameter.
+    //! letter in `alphabet()` with index \c i. The rules added are \f$a_ib_i =
+    //! e\f$ where the alphabet is \f$\{a_1, \ldots, a_n\}\f$; the 2nd parameter
+    //! \p vals is \f$\{b_1, \ldots, b_n\}\f$; and \f$e\f$ is the 3rd parameter.
     //!
     //! \tparam W the type of the words in the presentation
     //! \param p the presentation to add rules to
@@ -613,7 +632,7 @@ namespace libsemigroups {
     //! * \f$e ^ {-1} = e\f$ does not hold
     //!
     //! \complexity
-    //! \f$O(n)\f$ where \f$n\f$ is alphabet().size().
+    //! \f$O(n)\f$ where \f$n\f$ is `alphabet().size()`.
     template <typename W>
     void add_inverse_rules(Presentation<W>&                      p,
                            W const&                              vals,
@@ -622,9 +641,10 @@ namespace libsemigroups {
     //! Add rules for inverses.
     //!
     //! The letter in \c a with index \c i in \p vals is the inverse of the
-    //! letter in alphabet() with index \c i. The rules added are \f$a_ib_i =
-    //! e\f$ where the alphabet is \f$\{a_1, \ldots, a_n\}\f$; the parameter \p
-    //! vals is \f$\{b_1, \ldots, b_n\}\f$; and \f$e\f$ is the 3rd parameter.
+    //! letter in `alphabet()` with index \c i. The rules added are \f$a_ib_i =
+    //! e\f$ where the alphabet is \f$\{a_1, \ldots, a_n\}\f$; the 2nd
+    //! parameter \p vals is \f$\{b_1, \ldots, b_n\}\f$; and \f$e\f$ is the 3rd
+    //! parameter.
     //!
     //! \tparam W the type of the words in the presentation
     //! \param p the presentation to add rules to
@@ -725,6 +745,18 @@ namespace libsemigroups {
     template <typename W>
     void sort_rules(Presentation<W>& p);
 
+    //! Check if the rules \f$u_1 = v_1, \ldots, u_n = v_n\f$ satisfy \f$u_1v_1
+    //! < \cdots < u_nv_n\f$ where \f$<\f$ is the shortlex order.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //! \param p the presentation
+    //!
+    //! \returns A value of type `bool`.
+    //!
+    //! \throws LibsemigroupsException if `p.rules.size()` is odd.
+    template <typename W>
+    bool are_rules_sorted(Presentation<W> const& p);
+
     //! Returns the longest common subword of the rules.
     //!
     //! If it is possible to find a subword \f$w\f$ of the rules \f$u_1 = v_1,
@@ -766,6 +798,46 @@ namespace libsemigroups {
               typename = std::enable_if_t<!std::is_same<T, W>::value>>
     void replace_subword(Presentation<W>& p, T first, T last);
 
+    //! Replace non-overlapping instances of a subword via const reference.
+    //!
+    //! If \f$w=\f$`[first, last)` is a word, then this function replaces every
+    //! non-overlapping instance (from left to right) of \f$w\f$ in every rule,
+    //! adds a new generator \f$z\f$, and the rule \f$w = z\f$. The new
+    //! generator and rule are added even if \f$w\f$ is not a subword of any
+    //! rule.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \param p the presentation
+    //! \param w the subword to replace
+    //!
+    //! \returns (None)
+    //!
+    //! \throws LibsemigroupsException if \p w is empty.
+    // TODO(later) complexity
+    template <typename W>
+    void replace_subword(Presentation<W>& p, W const& w) {
+      replace_subword(p, w.cbegin(), w.cend());
+    }
+
+    //! Replace non-overlapping instances of a subword via `char const*`.
+    //!
+    //! If \f$w=\f$`[first, last)` is a word, then replaces every
+    //! non-overlapping instance (from left to right) of \f$w\f$ in every rule,
+    //! adds a new generator \f$z\f$, and the rule \f$w = z\f$. The new
+    //! generator and rule are added even if \f$w\f$ is not a subword of any
+    //! rule.
+    //!
+    //! \param p the presentation
+    //! \param w the subword to replace
+    //!
+    //! \returns (None)
+    //!
+    //! \throws LibsemigroupsException if \p w is empty.
+    inline void replace_subword(Presentation<std::string>& p, char const* w) {
+      replace_subword(p, w, w + std::strlen(w));
+    }
+
     //! Replace non-overlapping instances of a subword by another word.
     //!
     //! If \p existing and \p replacement are words, then this function
@@ -786,28 +858,33 @@ namespace libsemigroups {
                          W const&         existing,
                          W const&         replacement);
 
-    //! Replace non-overlapping instances of a subword via const reference.
+    //! Replace non-overlapping instances of a subword by another word.
     //!
-    //! If \f$w=\f$`[first, last)` is a word, then replaces every
-    //! non-overlapping instance (from left to right) of \f$w\f$ in every rule,
-    //! adds a new generator \f$z\f$, and the rule \f$w = z\f$. The new
-    //! generator and rule are added even if \f$w\f$ is not a subword of any
-    //! rule.
+    //! Adds the rule with left hand side `[lhs_begin, lhs_end)` and
+    //! right hand side `[rhs_begin, rhs_end)` to the rules.
+    //!
+    //! \tparam S the type of the first two parameters (iterators, or pointers)
+    //! \tparam T the type of the second two parameters (iterators, or pointers)
     //!
-    //! \tparam W the type of the words in the presentation
-    //! \tparam T the type of the 2nd and 3rd parameters (iterators)
     //! \param p the presentation
-    //! \param w the subword to replace
+    //! \param first_existing an iterator pointing to the first letter of the
+    //! existing subword to be replaced
+    //! \param last_existing an iterator pointing one past the last letter of
+    //! the existing subword to be replaced
+    //! \param first_replacement an iterator pointing to the first letter of
+    //! the replacement word
+    //! \param last_replacement an iterator pointing one past the last letter
+    //! of the replacement word
     //!
     //! \returns (None)
     //!
-    //! \exceptions
-    //! \no_libsemigroups_except
-    // TODO(later) complexity
-    template <typename W>
-    void replace_subword(Presentation<W>& p, W const& w) {
-      replace_subword(p, w.cbegin(), w.cend());
-    }
+    //! \throws LibsemigroupsException if `first_existing == last_existing`.
+    template <typename W, typename S, typename T>
+    void replace_subword(Presentation<W>& p,
+                         S                first_existing,
+                         S                last_existing,
+                         T                first_replacement,
+                         T                last_replacement);
 
     //! Replace instances of a word on either side of a rule by another word.
     //!
@@ -878,6 +955,41 @@ namespace libsemigroups {
     template <typename W>
     void normalize_alphabet(Presentation<W>& p);
 
+    //! Change or re-order the alphabet.
+    //!
+    //! This function replaces `p.alphabet()` with \p new_alphabet, where
+    //! possible, and re-writes the rules in the presentation using the new
+    //! alphabet.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \param p the presentation
+    //! \param new_alphabet the replacement alphabet
+    //!
+    //! \returns (None).
+    //!
+    //! \throws LibsemigroupsException if the size of `p.alphabet()` and \p
+    //! new_alphabet do not agree.
+    template <typename W>
+    void change_alphabet(Presentation<W>& p, W const& new_alphabet);
+
+    //! Change or re-order the alphabet.
+    //!
+    //! This function replaces `p.alphabet()` with \p new_alphabet where
+    //! possible.
+    //!
+    //! \param p the presentation
+    //! \param new_alphabet the replacement alphabet
+    //!
+    //! \returns (None).
+    //!
+    //! \throws LibsemigroupsException if the size of `p.alphabet()` and \p
+    //! new_alphabet do not agree.
+    inline void change_alphabet(Presentation<std::string>& p,
+                                char const*                new_alphabet) {
+      change_alphabet(p, std::string(new_alphabet));
+    }
+
     //! Returns an iterator pointing at the left hand side of the first
     //! rule of maximal length in the given range.
     //!
@@ -1025,6 +1137,121 @@ namespace libsemigroups {
     //! \throws LibsemigroupsException if `p.rules.size()` is odd.
     template <typename W>
     void remove_redundant_generators(Presentation<W>& p);
+
+    //! Return a possible letter by index.
+    //!
+    //! This function returns the \f$i\f$-th letter in the alphabet consisting
+    //! of all possible letters of type Presentation<W>::letter_type. This
+    //! function exists so that visible ASCII characters occur before invisible
+    //! ones, so that when manipulating presentations over `std::string`s the
+    //! human readable characters are used before non-readable ones.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \returns A `letter_type`.
+    //!
+    //! \throws LibsemigroupsException if `i` exceeds the number of letters in
+    //! supported by `letter_type`.
+    template <typename W>
+    typename Presentation<W>::letter_type letter(Presentation<W> const&,
+                                                 size_t i);
+
+    //! Return a possible letter by index.
+    //!
+    //! This function returns the \f$i\f$-th letter in the alphabet consisting
+    //! of all possible letters of type Presentation<std::string>::letter_type.
+    //! This function exists so that visible ASCII characters occur before
+    //! invisible ones, so that when manipulating presentations over
+    //! `std::string`s the human readable characters are used before
+    //! non-readable ones.
+    //!
+    //! \param p a presentation (unused)
+    //! \param i the index
+    //!
+    //! \returns A `letter_type`.
+    //!
+    //! \throws LibsemigroupsException if \p i exceeds the number of letters in
+    //! supported by `letter_type`.
+    //!
+    //! \sa `character(size_t)`
+    template <>
+    inline typename Presentation<std::string>::letter_type
+    letter(Presentation<std::string> const& p, size_t i);
+
+    //! Return a possible character by index.
+    //!
+    //! This function returns the \f$i\f$-th letter in the alphabet consisting
+    //! of all possible letters of type Presentation<std::string>::letter_type.
+    //! This function exists so that visible ASCII characters occur before
+    //! invisible ones, so that when manipulating presentations over
+    //! `std::string`s the human readable characters are used before
+    //! non-readable ones.
+    //!
+    //! \param i the index
+    //!
+    //! \returns A `letter_type`.
+    //!
+    //! \throws LibsemigroupsException if \p i exceeds the number of letters in
+    //! supported by `letter_type`.
+    //!
+    //! \sa `letter(Presentation<std::string> const&, size_t)`
+    inline typename Presentation<std::string>::letter_type character(size_t i);
+
+    //! Returns the first letter **not** in the alphabet of a presentation.
+    //!
+    //! This function returns `letter(p, i)` when `i` is the least possible
+    //! value such that `!p.in_alphabet(letter(p, i))` if such a letter exists.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \param p the presentation
+    //!
+    //! \returns A `letter_type`.
+    //!
+    //! \throws LibsemigroupsException if \p p already has an alphabet of
+    //! the maximum possible size supported by `letter_type`.
+    template <typename W>
+    typename Presentation<W>::letter_type
+    first_unused_letter(Presentation<W> const& p);
+
+    //! Convert a monoid presentation to a semigroup presentation.
+    //!
+    //! This function modifies its argument in-place by replacing the empty
+    //! word in all relations by a new generator, and the identity rules for
+    //! that new generator. If `p.contains_empty_word()` is `false`, then the
+    //! presentation is not modified and \ref UNDEFINED is returned. If a new
+    //! generator is added as the identity, then this generator is returned.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \param p the presentation
+    //!
+    //! \returns The new generator added, if any, and \ref UNDEFINED if not.
+    //!
+    //! \throws LibsemigroupsException if `replace_word` or
+    //!  `add_identity_rules` does.
+    template <typename W>
+    typename Presentation<W>::letter_type make_semigroup(Presentation<W>& p);
+
+    //! Greedily reduce the length of the presentation using
+    //! `longest_common_subword`.
+    //!
+    //! This function repeatedly calls `longest_common_subword` and
+    //! `replace_subword` to introduce a new generator and reduce the length of
+    //! the presentation \p p until `longest_common_subword` returns the empty
+    //! word.
+    //!
+    //! \tparam W the type of the words in the presentation
+    //!
+    //! \param p the presentation
+    //!
+    //! \returns (None)
+    //!
+    //! \throws LibsemigroupsException if `longest_common_subword` or
+    //!  `replace_word` does.
+    template <typename W>
+    void greedy_reduce_length(Presentation<W>& p);
+
   }  // namespace presentation
 }  // namespace libsemigroups
 
