@@ -27,12 +27,13 @@
 #include <algorithm>      // for equal, find_if
 #include <cstddef>        // for size_t
 #include <cstdint>        // for uint64_t
+#include <functional>     // for hash
 #include <iosfwd>         // for string
 #include <iterator>       // for operator+
 #include <map>            // for map
 #include <stack>          // for stack
 #include <unordered_map>  // for unordered_map
-#include <utility>        // for hash
+#include <utility>        // for pair, make_pair
 #include <vector>         // for vector
 
 #include "adapters.hpp"   // for Hash, EqualTo
@@ -119,8 +120,12 @@ namespace libsemigroups {
       struct State {
         node_index_type v;
         edge_index_type pos;
-        State()             = default;
-        State(State const&) = default;
+        State()                        = default;
+        State(State const&)            = default;
+        State(State&&)                 = default;
+        State& operator=(State const&) = default;
+        State& operator=(State&&)      = default;
+
         State(node_index_type vv, edge_index_type ppos) : v(vv), pos(ppos) {}
 
         bool valid() const noexcept {
@@ -479,63 +484,6 @@ namespace libsemigroups {
     };
 
     class DFSHelper {
-     public:
-      using const_iterator = typename detail::SuffixTree::const_iterator;
-      explicit DFSHelper(detail::SuffixTree& st)
-          : _best(),
-            _best_goodness(),
-            _distance_from_root(st.number_of_nodes(), 0),
-            _num_leafs(st.number_of_nodes(), 0),
-            _suffix_index() {}
-
-      void pre_order(detail::SuffixTree const& st, size_t v) {
-        // This is a tree so we've never seen v before!
-        if (!st.is_root(v)) {
-          _distance_from_root[v]
-              = _distance_from_root[st.parent(v)] + st.length(v);
-        }
-        if (st.is_leaf(v)) {
-          _num_leafs[v]++;
-          // Starting index of the suffix that the leaf corresponds to
-          _suffix_index.push_back(st.right(v) - _distance_from_root[v]);
-        }
-      }
-
-      void post_order(detail::SuffixTree const& st, size_t v) {
-        if (st.is_leaf(v)) {
-          return;
-        }
-
-        for (auto const& child : st.node(v).children) {
-          _num_leafs[v] += _num_leafs[child.second];
-        }
-        _scratch.assign(_suffix_index.cend() - _num_leafs[v],
-                        _suffix_index.cend());
-        std::sort(_scratch.begin(), _scratch.end());
-        // number of non-overlapping subwords corresponding to the node v
-        size_t num_non_overlap = st.multiplicity(_scratch[0]);
-        for (size_t i = 0; i < _scratch.size() - 1; ++i) {
-          if (_scratch[i] + _distance_from_root[v] <= _scratch[i + 1]) {
-            num_non_overlap += st.multiplicity(_scratch[i]);
-          }
-        }
-        int goodness = (_distance_from_root[v] * num_non_overlap)
-                       - num_non_overlap - (_distance_from_root[v] + 1);
-        if (goodness > _best_goodness) {
-          _best          = v;
-          _best_goodness = goodness;
-        }
-      }
-
-      std::pair<const_iterator, const_iterator>
-      yield(detail::SuffixTree const& st) {
-        if (st.is_root(_best)) {
-          return st.word(0, 0);
-        }
-        return st.word(st.left(_best) - _distance_from_root[st.parent(_best)],
-                       st.right(_best));
-      }
-
      private:
       size_t              _best;
       int                 _best_goodness;
@@ -543,6 +491,17 @@ namespace libsemigroups {
       std::vector<size_t> _num_leafs;
       std::vector<size_t> _scratch;
       std::vector<size_t> _suffix_index;
+
+     public:
+      using const_iterator = typename detail::SuffixTree::const_iterator;
+      explicit DFSHelper(detail::SuffixTree& st);
+
+      void pre_order(detail::SuffixTree const& st, size_t v);
+
+      void post_order(detail::SuffixTree const& st, size_t v);
+
+      std::pair<const_iterator, const_iterator>
+      yield(detail::SuffixTree const& st);
     };
 
     namespace suffix_tree_helper {
